
PROGRAM elastic_1D

!! DK DK SEM 1D elastic by Bernhard Schuberth, Munich, September 2003

!Program to solve the one-dimensional wave equation with a spectral element method

!written by Bernhard Schuberth 2003
!in the course of the diploma thesis
!"The Spectral Element Method for Seismic Wave Propagation - Theory,Implementation and
!Comparison to Finite Difference Methods"
!!at the Departement for Earth- and Environmental Sciences, Geophysics Section, of the
!Ludwig-Maximilians-University Munich

!___________________________________________________________________________________________________

!modules used

!! DK DK PGPLOT Version 5.2.0 is available from Caltech at
!! DK DK ftp://astro.caltech.edu/pub/pgplot/pgplot5.2.tar.gz
use plot_mod      !plotting tools based on pgplot
use el_spec_plot    !plotting subroutines needed for this program
use source      !creates different source time functions
use write_data      !used for writing seismograms to outputfiles
use el_array      !specific arrays for comparison with optimal FD operators
use sem_spec_modules    !specific subroutines needed for SEM
use jacobian_mod    !calculation of the Jacobi-Matrix and Jacobian
use shape_mod     !calculation of shape functions of the mapping
use sem_analyt_mod    !calculation of the corresponding analytical solution
use signal_processing   !subroutines for fft and convolution
use minmax1d      !maxi/min of a vector, used because of problems in "minval"
use minmax      !maximum or minimum of 2-D or 3-D matrices

!___________________________________________________________________________________________________


implicit none

double precision, dimension(:,:), allocatable :: me                         !elemental mass matrices
double precision, dimension(:,:), allocatable :: DL             !derivatives of Lagrange polynomials
double precision, dimension(:,:), allocatable :: rho_e                      !density in all elements
double precision, dimension(:,:), allocatable :: mue_e    !elasticity coefficient mu in all elements
double precision, dimension(:,:), allocatable :: ve           !wave velocity derived from rho and mu

double precision, dimension(:,:), allocatable :: greens                      !stores all seismograms
double precision, dimension(:,:), allocatable :: analytical  !all corresp. analytical solutions (AS)

double precision, dimension(:,:), allocatable :: Jacobi           !Jacobi-Matrix (J-M)of the mapping
double precision, dimension(:,:), allocatable :: Jacobian                        !determinant of J-M
double precision, dimension(:,:), allocatable :: Jacobi_inverse          !J-M of the inverse mapping
double precision, dimension(:,:), allocatable :: shape1D             !shape functions of the mapping
double precision, dimension(:,:), allocatable :: dershape1D       !derivatives of the shap functions

double precision, dimension(:), allocatable :: U          !displacement at global nodes (timestep i)
double precision, dimension(:), allocatable :: Unew     !displacement at global nodes (timestep i+1)
!! DK DK single precision version for display
real, dimension(:), allocatable :: Unew_sngl
double precision, dimension(:), allocatable :: Uold     !displacement at global nodes (timestep i-1)
double precision, dimension(:), allocatable :: U_init             !initial displacement field at t=0
double precision, dimension(:), allocatable :: Uold_init        !initial displacement field at t= -1
double precision, dimension(:), allocatable :: Mass         !global mass matrix (stored as a vector)
double precision, dimension(:), allocatable :: MassInv                   !inverse of the mass matrix
double precision, dimension(:), allocatable :: F                             !forces at global nodes
double precision, dimension(:), allocatable :: Rho_global    !globally defined density needed for AS
double precision, dimension(:), allocatable :: Mue_global !globally defined elasticity coeff. for AS

double precision, dimension(:), allocatable :: x                      !x-vector (spatial coordinate)
!! DK DK single precision version for display
real, dimension(:), allocatable :: x_sngl
double precision, dimension(:), allocatable :: dx                  !distance between global x points
double precision, dimension(:), allocatable :: xmesh   !defining the 1-D mesh (2 points per element)
double precision, dimension(:), allocatable :: de                            !length of the elements
double precision, dimension(:), allocatable :: xi         !GLL quadrature points (local coordinates)
double precision, dimension(:), allocatable :: xidiff                   !distance between GLL points
double precision, dimension(:), allocatable :: weights    !weights of the GLL integration quadrature

double precision, dimension(:), allocatable :: t                                        !time vector
double precision, dimension(:), allocatable :: src          !source time function for the simulation
double precision, dimension(:), allocatable :: stf             !source time function for convolution

double precision, dimension(:), allocatable :: seismogram        !convolved seismogram for plotting
double precision, dimension(:), allocatable :: conv_ana                  !convolved AS for plotting
double precision, dimension(:), allocatable :: diff_ana    !error in AS <= discrete sampling in time
double precision, dimension(:), allocatable :: rectimes            !special time marks needed for AS
double precision, dimension(:), allocatable :: reftimes            !special time marks needed for AS
double precision, dimension(:), allocatable :: srd       !source receiver distance for each receiver
double precision, dimension(:), allocatable :: receiver_pos              !x coordinates of receivers
double precision, dimension(:), allocatable :: receiver_dx !distance of receiver to nearest gridnode

double precision srd_diff                 !distances between receivers

double precision  dt                                                          !duration of time step
double precision  dxmin                                                         !minimum value of dx
double precision  v_max                                                         !maximum value of ve
double precision  v_min                                                         !minimum value of ve
double precision  pt                   !period of the dominant frequency of a wavelet given by 1/frq

double precision  source_pos                                             !x coordinate of the source
double precision  diff_greens !error in seismogram due to discrete time-sampling (only for plotting)

double precision Length                                                         !Length of the model
double precision dx_mean                                     !average of distance between grid nodes

double precision bv1, bvng            !values used for calculation of absorbing boundaries at (1,ng)
double precision maxU, minU            !maximum and minimum values of displacement used for plotting
double precision src_max, src_min      !maximum and minimum values of displacement used for plotting
double precision tmp                                                                 !dummy variable


!Connectivity-Matrix
integer,dimension(:,:), allocatable :: C

integer,dimension(:), allocatable :: receiver_ix
integer,dimension(:), allocatable :: num_el

integer, dimension(1:2) :: el_step_ix

!polyn. order,number of elements,number of grid points
integer N,ne,ng
integer M,nt, source_ix, rec_num, dngix,step_ix
!indices
integer i,j

character(len=2) deg                                                                   !string for N
!used for correct numbering of gif-output
character(len=3) gif_num
!input files
character(len=50) mue_file, rho_file
!output files
character(len=50) fname1, setupfile,plot_file
!strings used for plotting
character(len=50) poldeg,xlab, ylab, time


!Definition of fixed Parameters:

integer, parameter :: NGNOD = 2 !number of anchor nodes used for the shape functions of the elements

!___________________________________________________________________________________________________

!variables that can be changed by the user by editing the file "parfile" (see also for descriptions)

double precision tmax                              !the duration of wave propagation to be simulated
double precision Courant_nr                              !Courant number for the stability criterion
double precision frq                    !dominant frequency of wavelet to be simulated or convolved
double precision distance                               !distance of receiver, when only one is used

integer pd                                                                              !plot device
integer src_ne                                                  !number of element to contain source
integer src_N                                        !GLL point at which the source shall be located
integer plot_rec_nr                                      !plot the seismogram of receiver number ...

character(len=2) mod_id                                                         !flag for model type
character(len=2) stfpar                                                        !flag for source type
integer receiver_id                                                       !flag for array type
character(len=1) bcondition                                            !flag for boundary conditions
logical convolve                                              !flag for convolution
logical txt                                                          !flag for creating ascii output
logical percent                                             !flag for run-time display of percentage
logical src_flag                                         !flag if point source or initial conditions
logical reg_grid_flag                                             !flag if regular or irregular grid

!end of initialization


!___________________________________________________________________________________________________


!Read parameter file "./parfile"

open(unit=19,file='parfile')
  read(19,*) N
  read(19,*) reg_grid_flag
  read(19,*) ne
  read(19,*) mod_id
  read(19,*) tmax
  read(19,*) Length
  read(19,*) Courant_nr
  read(19,*) bcondition
  read(19,*) src_flag
  read(19,*) src_ne
  read(19,*) src_N
  read(19,*) stfpar
  read(19,*) frq
  read(19,*) receiver_id
  read(19,*) distance
  read(19,*) pd
  read(19,*) plot_rec_nr
  read(19,*) convolve
  read(19,*) percent
  read(19,*) txt
close(19)

!! DK DK check parameters read
  write(*,*) 'Control of parameters read from input file:'
  write(*,*)
  write(*,*) N
  write(*,*) reg_grid_flag
  write(*,*) ne
  write(*,*) mod_id
  write(*,*) tmax
  write(*,*) Length
  write(*,*) Courant_nr
  write(*,*) bcondition
  write(*,*) src_flag
  write(*,*) src_ne
  write(*,*) src_N
  write(*,*) stfpar
  write(*,*) frq
  write(*,*) receiver_id
  write(*,*) distance
  write(*,*) pd
  write(*,*) plot_rec_nr
  write(*,*) convolve
  write(*,*) percent
  write(*,*) txt

  if(N >= 100) stop 'Degree N >= 100 is not allowed'
  if(receiver_id /= 0 .and. receiver_id /= 1) stop 'receiver_id must be either 0 or 1'

!get the source into the correct element given by "src_ne"

if (src_N > N) then
do while (src_N > N)
src_N=src_N-N
enddo
endif

mod_id = trim(mod_id)

deg = nletterstring(N,2)

!dominant period of the wavelet

pt=1./frq

!create filename for this run
!----------------------------

fname1=''

if (mod_id == 'so') then
  call create_sem_fname(fname1,'eo_s',N,2,ne,5,receiver_id)
elseif (mod_id == 'st') then
  call create_sem_fname(fname1,'et_s',N,2,ne,5,receiver_id)
elseif (mod_id == 'sh') then
  call create_sem_fname(fname1,'eh_s',N,2,ne,5,receiver_id)
endif

!check for irregular grid
!------------------------

if (reg_grid_flag) then

  allocate(xmesh(0:ne),de(1:1))
  xmesh(0)=0
  de(1)=1./float(ne)*Length
  do i=1,ne
    xmesh(i)=xmesh(i-1) + de(1)
  enddo
else

!if irregular grid get mesh data form file "./DATA_mesh/mesh1D.dat"

  open(unit=24,file='DATA_mesh/mesh1D.dat')
i=0
  do
  read(24,*,end=500)
  i=i+1
  enddo
  500 close(24)

  ne=i-1
  allocate(xmesh(0:ne),de(1:ne))

  open(unit=24,file='DATA_mesh/mesh1D.dat')
i=0
  do
  read(24,*,end=700) xmesh(i)
  i=i+1
  enddo
  700 close(24)

  do i=1,ne
  de(i)=xmesh(i)-xmesh(i-1)
  enddo

  Length=xmesh(ne)-xmesh(0)
endif

!___________________________________________________________________________________________________

!starting the program
!--------------------

!calculating the total number of nodes ng (global numbering)

ng = N * ne + 1

M = N + 1

!allocating all matrices

allocate(me(0:N,1:ne),rho_e(0:N,1:ne),mue_e(0:N,1:ne),ve(0:N,1:ne))
allocate(Mass(1:ng),MassInv(1:ng),U(1:ng),F(1:ng),x(1:ng),Uold(1:ng),Unew(1:ng),dx(1:ng-1))
allocate(Rho_global(1:ng),Mue_global(1:ng))
allocate(weights(0:N),DL(0:N,0:N),C(1:M,1:ne),xi(0:N),xidiff(1:N))
allocate(U_init(1:ng),Uold_init(1:ng))
allocate(Jacobi(0:N,1:ne),Jacobian(0:N,1:ne),Jacobi_inverse(0:N,1:ne))
allocate(shape1D(1:NGNOD,0:N),dershape1D(1:NGNOD,0:N))

!! DK DK allocate single precision version for display
allocate(x_sngl(1:ng),Unew_sngl(1:ng))

!clearing all variables

Mass=0
MassInv=0
U=0
me=0
F=0
DL=0
Jacobi=0
Jacobian=0
Jacobi_inverse=0
shape1D=0
dershape1D=0
C=0
Uold=0
Unew=0
Rho_global=0
Mue_global=0
rho_e=0
mue_e=0
minU=huge(U)
maxU=tiny(U)


!read the information on elasticity coefficient mu from file
!-----------------------------------------------------------

if (mod_id == 'sh') then
  mue_file='DATA_mue/'//fname1(1:12)//'_mue.txt'
  open(unit=25,file=mue_file)
  i=0
  do i=1,ne
     do j=0,N
    read(25,*) mue_e(j,i)
     enddo
  enddo
elseif (mod_id == 'so') then
  mue_file='DATA_mue/'//fname1(1:4)//'00_00hom_mue.txt'
  open(unit=25,file=mue_file)
  read(25,*) mue_e(0,1)
  mue_e = mue_e(0,1)
  step_ix = 0
  Mue_global = mue_e(0,1)

elseif (mod_id == 'st') then
  mue_file='DATA_mue/'//fname1(1:4)//'00_00two_mue.txt'
  open(unit=25,file=mue_file)
  read(25,*) mue_e(0,1)
  read(25,*) mue_e(N,ne)
  read(25,*) el_step_ix(1)
  mue_e(:,1:el_step_ix(1)-1) = mue_e(0,1)
  mue_e(:,el_step_ix(1):ne) = mue_e(N,ne)
  step_ix = N * (el_step_ix(1)-1) + 1

  !creating the "global" Mue-vector
  !(only needed for the calcualtion of the analytical solution)!
  !-------------------------------------------------------------

  Mue_global(1:step_ix-1) = mue_e(0,1)
  Mue_global(step_ix:ng) = mue_e(N,ne)

endif
write(*,*)
write(*,*) '**************** Finished reading mue data! *****************'
close(25)

!read the density information from file
!--------------------------------------

if (mod_id == 'sh') then
  rho_file='DATA_rho/'//fname1(1:12)//'_rho.txt'
  open(unit=26,file=rho_file)
  i=0
  do i=1,ne
     do j=0,N
    read(26,*) rho_e(j,i)
     enddo
  enddo
elseif (mod_id == 'so') then
  rho_file='DATA_rho/'//fname1(1:4)//'00_00hom_rho.txt'
  open(unit=26,file=rho_file)
  read(26,*) rho_e(0,1)
  rho_e = rho_e(0,1)
  step_ix = 0
  Rho_global = rho_e(0,1)

elseif (mod_id == 'st') then
  rho_file='DATA_rho/'//fname1(1:4)//'00_00two_rho.txt'
  open(unit=25,file=rho_file)
  read(25,*) rho_e(0,1)
  read(25,*) rho_e(N,ne)
  read(25,*) el_step_ix(2)
  if (el_step_ix(1) /= el_step_ix(2)) then
    write(*,*) '*************************************************************'
    write(*,*) '***                       Warning!                        ***'
    write(*,*) '***        Properties change in different elements        ***'
    write(*,*) '***              Not a two layer case anymore             ***'
    write(*,*) '*** Calculation of an analytical solution is not possible!***'
    write(*,*) '*************************************************************'
  endif
  rho_e(:,1:el_step_ix(2)-1) = rho_e(0,1)
  rho_e(:,el_step_ix(2):ne) = rho_e(N,ne)
  step_ix = N * (el_step_ix(2)-1) + 1

  !creating the "global" Rho-vector
  !(only needed for the calcualtion of the analytical solution)!
  !-------------------------------------------------------------

  Rho_global(1:step_ix-1) = rho_e(0,1)
  Rho_global(step_ix:ng) = rho_e(N,ne)

endif
write(*,*)
write(*,*) '************** Finished reading density data! ***************'
close(26)

!calculating the wave velocity inside each element
!-------------------------------------------------

do i=1,ne
   do j=0,N
  ve(j,i)=sqrt(mue_e(j,i)/rho_e(j,i))
   enddo
enddo

write(*,*)
write(*,*) 'step_ix = ', step_ix


!calculating the derivatives of the lagrange polynomials at the gll nodes
!and getting the collocation points xi(i) and the weights weights(i) for
!the gll integration rule
!------------------------------------------------------------------------

call diff_lagrange(N,DL,weights,xi)


!defining the mapping function and the Jacobi matrix
!----------------------------------------------------

call get_shape1D(shape1D,dershape1D,xi)

call calc_jacobian1D(Jacobi,Jacobian,Jacobi_inverse,shape1D,dershape1D,xmesh)

!creating the correct x vector with non equidistant points

call create_x_irreg(x,xmesh,shape1D)

do i=1,ng-1
  dx(i)=x(i+1)-x(i)
enddo

!defining the time vector
!------------------------

  !getting the stability criterion right!

  !checking if Courant number is small enough for absorbing and periodic boundary conditions

  if (bcondition == 'a') then
    if (Courant_nr > 0.42) then
    Courant_nr = 0.42
    write(*,*)
    write(*,*)
    write(*,*) '**************************** WARNING! ****************************'
    write(*,*)
    write(*,*) '** Courant value is too high for absorbing boundary conditions! **'
    write(*,*)
    write(*,*) '** Changing the Courant value to 0.42 (maximum value possible)! **'
    write(*,*)
    write(*,*) '******************************************************************'
    write(*,*)
    endif
  endif


  dxmin = min_vector(dx)

  v_max=max_matrix2d(ve)

  dt = Courant_nr * dxmin/v_max          !Stability Criterion!!!!!!!!!!!!!

  write(*,*)
  write(*,*) 'dt = ', dt

  !calculating the number of time steps to be evaluated

  nt = idint(tmax / dt)

  !allocating all vectors, which length is dependent on nt

  allocate(t(1:nt),src(1:nt),stf(1:nt),rectimes(1:nt),reftimes(1:nt))

  t=0
  src=0

  !time vector

  t = (/ (float(i), i=1,nt) /)*dt

!defining the source term or the initial conditions, depending on src_flag
!-------------------------------------------------------------------------


if (src_flag) then                         !point source is used

  !creating two source time functions: delta peak in time for
  !the solution of the wave equation => Greens function of the
  !model space
  !and a ricker wavelet with dominant period pt which is later
  !convolved with the Greens function to obtain a synthetic
  !seismogram

  if (stfpar == 'ga') then
  call gauss(src,dt,pt)
  stf=src
  elseif (stfpar == 'r1') then
  call ricker(src,dt,pt,nt)
  stf=src
  elseif (stfpar == 'r2') then
    call ricker2(src,dt,pt,nt)
  stf=src
  elseif (stfpar == 'de') then
  call delta(nt,1,dt,src)
    if (pd /= 3) then
    call ricker2(stf,dt,pt,nt)
    endif

  endif

else                                          !initial conditions are used

  open(unit=19,file='DATA_initial/'//fname1(1:13)//'in1.txt')
  do i=1,ng
  read(19,*) U_init(i)
  enddo
  close(19)

  open(unit=19,file='DATA_initial/'//fname1(1:13)//'in2.txt')
  do i=1,ng
  read(19,*) Uold_init(i)
  enddo
  close(19)

  U=U_init
  Uold=Uold_init

endif


! creating the elemental mass matrices me
!----------------------------------------

  do i=1,ne
     do j=0,N
      me(j,i) = rho_e(j,i) * Jacobian(j,i) * weights(j)
     enddo
  enddo


!creating the connectivity matrix for the 1D case
!------------------------------------------------

C=0

do i=1,ne
    do j=1,M
        C(j,i)=j+(i-1)*N
    enddo
enddo


!connecting the vectors me to M
!-----------------------------

call connect_vec(N,ne,ng,me,Mass,C)

!if periodic boundary conditions are used connect first and last element

if (bcondition == 'p') then
  tmp=0.
  tmp=Mass(1)
  Mass(1)=Mass(1)+Mass(ng)
  Mass(ng)=Mass(ng)+tmp
endif


!Adding the boundary conditions to the source vector F
!-> see subroutine calculate_F in module sem_spec_modules.f90


!setting up the receiver array according to receiver_id
!---------------------------------------------------

!calculating the source position

  source_ix = (src_ne-1)*N + src_N
  source_pos = x(source_ix)

!calculating the receiver positions

  rec_num = 1
  plot_rec_nr = 1

  allocate(receiver_ix(1:rec_num),srd(1:rec_num))
  allocate(receiver_pos(1:rec_num),num_el(1:rec_num))
  allocate(receiver_dx(1:rec_num))

    srd = 0.
    srd_diff = 0.
    receiver_pos = 0.
    receiver_ix = 0
    receiver_dx = 0.

  srd(1) = distance
  call setup_ac_receiver(receiver_id,source_ix,receiver_ix,rec_num,srd,srd_diff,&
                            receiver_pos,receiver_dx,num_el,x,xmesh)

! allocating the seismogram

  allocate(greens(1:nt,1:rec_num))


  allocate(analytical(1:rec_num,1:nt),diff_ana(1:rec_num))
  allocate(conv_ana(1:nt))
  conv_ana=0.

!if plot_dev is not 3:
!preparing the plot
!--------------------

if (pd /= 3) then

  if (N >= 10) then
    poldeg = "Polynomials of degree "//deg(:)
  else
    poldeg = "Polynomials of degree "//deg(2:)
  endif

  xlab="x"
  xlab=trim(xlab)
  ylab="Displacement U"
  ylab=trim(ylab)

  analytical(1,nt/2:nt)=1.
  call conv_fft(analytical(1,:),stf,conv_ana)

  src_max=1.2*max_vector(conv_ana)
  src_min=1.2*min_vector(conv_ana)

endif


!calculating the solution to the wave equation
!---------------------------------------------

!inverting the mass matrix

do i=1,ng
   MassInv(i) = 1. / Mass(i)
enddo


!***************************************** time loop ***********************************************
!---------------------------------------------------------------------------------------------------

   write(*,*)
   write(*,*) '******************** Time loop started! *********************'
   write(*,*)

do i=1,nt

   !compute the force at every grid point of the mesh for timestep i

   F=0

   if (bcondition == 'a') then                                 !absorbing boundaries
  bv1=rho_e(0,1)*ve(0,1)*((U(1)-Uold(1))/dt)
  bvng=rho_e(N,ne)*ve(N,ne)*((U(ng)-Uold(ng))/dt)

  call calculate_F(N,ne,DL,weights,mue_e,U,F,C,Jacobian,Jacobi_inverse,ve(src_N,src_ne),&
                         src(i),src_ne,src_N,bv1,bvng,stfpar,dt)
   else

  call calculate_F(N,ne,DL,weights,mue_e,U,F,C,Jacobian,Jacobi_inverse,ve(src_N,src_ne),&
                         src(i),src_ne,src_N,stfpar=stfpar,dt=dt)
   endif

   if (bcondition == 'p') then
     tmp=0.
     tmp=F(1)
     F(1)=F(1)+F(ng)
     F(ng)=F(ng)+tmp
   endif

   !checking for boundary conditions

   if (bcondition == 'r') then

     Unew(2:ng-1) = dt**2. * MassInv(2:ng-1) * F(2:ng-1) + 2.*U(2:ng-1) - Uold(2:ng-1)

   else

     Unew = dt**2. * MassInv * F + 2.*U - Uold

   endif

   !creating the Greens functions

   do j=1,rec_num

  if (receiver_id == 0) then          !receivers somewhere inside the elements

    call get_func_at_pos(N,num_el(j),Unew,greens(i,j),receiver_pos(j),xi,C,xmesh)

  elseif (receiver_id == 1) then      !receivers exactly on GLL points
      greens(i,j) = Unew(receiver_ix(j))
  endif

   enddo


   !if plot_dev is not 3:
   !---------------------

   if (pd /= 3 .and. pd /= 4) then

    !preparing the plot

    write(time,'(F10.5)') t(i)
    time = trim(time)

    !plot the displacement at time (i) on the intervall [0,1]


  if (src_flag .and. stfpar /= 'de') then
  if (min_vector(Unew) < src_min) then
  else
    minU=src_min
  endif
  if (max_vector(Unew) > src_max) then
  else
    maxU=src_max
  endif
  else

  if (min_vector(Unew) < minU) minU=min_vector(Unew)
  if (max_vector(Unew) > maxU) maxU=max_vector(Unew)
  endif

!! DK DK convert to single precision for display
  x_sngl = real(x)
  Unew_sngl = real(Unew)

  if (pd == 1 .or. pd == 2) then
    if (i == 1) j=0
    if (i == 1 .or. mod(i,50) == 0) then
      j=i/50 + 1
      gif_num=nletterstring(j,3)
      plot_file=trim('DATA_TMP/'//trim(fname1(1:13))//'nr'//trim(gif_num))

      if (src_flag) then
        call plot_1d_snapshot(ng,x_sngl,Unew_sngl,xlab,ylab,poldeg,trim(plot_file),&
                                  pd,time,i,real(minU),real(maxU),real(source_pos))
      else
        call plot_1d_snapshot(ng,x_sngl,Unew_sngl,xlab,ylab,poldeg,trim(plot_file),&
                                  pd,time,i,real(minU),real(maxU))
      endif
    endif
  else
      if (src_flag) then
        call plot_1d_snapshot(ng,x_sngl,Unew_sngl,xlab,ylab,poldeg,'DATA_TMP/'//fname1,&
                                pd,time,i,real(minU),real(maxU),real(source_pos))
      else
        call plot_1d_snapshot(ng,x_sngl,Unew_sngl,xlab,ylab,poldeg,trim(plot_file),&
                                  pd,time,i,real(minU),real(maxU))
      endif
  endif
   elseif (pd == 4 .and. i == nt) then
  pd = 0
   endif


   !prepare for next time step

   Uold=U
   U=Unew

   if(mod(i,10) == 0) print *,'computing time step ',i,' out of ',nt

enddo

!*************************************** end of time loop ******************************************
!---------------------------------------------------------------------------------------------------

  write(*,*)
  write(*,*) '******************** Time loop ended! ***********************'
  write(*,*)

!calculate the corresponding analytical solution

call pgclos

if (pd /= 3 .or. txt) then

  !analytical solution only available for homogeneous or two layered model
  if (mod_id == 'so' .or. mod_id == 'st') then
  if (receiver_id == 1 .and. el_step_ix(1) == el_step_ix(2)) then
    call calcanalyt2lay_sem(analytical,receiver_ix,dt,dx,source_ix,Rho_global,Mue_global,src,&
          1,diff_ana,rectimes,reftimes)
  elseif (receiver_id == 0 .and. el_step_ix(1) == el_step_ix(2)) then
    call calcanalyt2lay_sem(analytical,receiver_ix,dt,dx,source_ix,Rho_global,Mue_global,src,&
          1,diff_ana,rectimes,reftimes,receiver_dx)
  endif
  endif
endif

!if plot_dev is not 3:
!---------------------

if (pd /= 3) then                                                     !only done when plotting is on

plot_file=trim('DATA_TMP/'//trim(fname1)//trim('_final'))

 !if point source was used
 if (src_flag) then

  if (mod_id == 'so' .or. mod_id == 'st') then
    conv_ana=0.
    call conv_fft(analytical(plot_rec_nr,:),stf,conv_ana)    !get convolved analytical solution

   !if simulation used delta peak in time then check for convolution-flag "convolve"
   if (stfpar == 'de' .and. convolve) then

  allocate(seismogram(1:nt))

  call conv_fft(greens(:,plot_rec_nr),stf,seismogram)   !get convolved seismogram at receiver

  !plot greens function, analytical greens function together with convolved seismogram and
  !analytical solution
  call plot_ac_seismogram(t,greens(:,plot_rec_nr),analytical(plot_rec_nr,:),&
                          plot_file,pd,seismogram,conv_ana,stf,&
                                diff_ana(plot_rec_nr),diff_greens)

if (pd == 2) call pgclos

   !if simulation used delta peak in time and convolution is not wanted
   elseif (stfpar == 'de' .and. convolve) then

  !plot only greens function and analytical greens function
  call plot_ac_seismogram(t=t,greens=greens(:,plot_rec_nr),heavyside=&
                                analytical(plot_rec_nr,:),plot_outfile=&
                          plot_file,pd=pd,diff_ana=diff_ana(plot_rec_nr))

   !if simulation used a band limited signal (ricker, gaussi, etc.)
   elseif (stfpar /= 'de') then

   !plot the simulated seismogram and the convolved corresponding analytical solution
  call plot_ac_seismogram(t=t,greens=greens(:,plot_rec_nr),heavyside=conv_ana(1:nt),&
        plot_outfile=plot_file,pd=pd,stf=stf,diff_ana=diff_ana(plot_rec_nr))

   endif

  else
   if (.not. convolve) then

  call plot_ac_seismogram(t=t,greens=greens(:,plot_rec_nr),heavyside=&
        (/(0.*dble(i),i=1,size(greens))/),plot_outfile=plot_file,&
        pd=pd,stf=stf)
   else

  allocate(seismogram(1:nt))
  call conv_fft(greens(:,plot_rec_nr),stf,seismogram)   !get convolved seismogram at receiver

  call plot_ac_seismogram(t=t,greens=greens(:,plot_rec_nr),heavyside=&
        (/(0.*dble(i),i=1,size(greens))/),plot_outfile=plot_file,&
        pd=pd,seismo=seismogram,stf=stf)
   endif
  endif

 !if initial conditions were used instead of a source
 else

        !plot initial conditions
  call plot_initial(x,U_init,Uold_init,plot_outfile=plot_file,pd=pd)

  !plot only seismogram, analytical solution is not considered here
  call plot_ac_seismogram(t=t,greens=greens(:,plot_rec_nr),&
                                heavyside=(/ (0.*dble(i),i=1,size(greens)) /),&
                          plot_outfile=plot_file,pd=pd)

 endif

endif


!output file wanted?
!-------------------

if (txt) then

  write(*,*)
  write(*,*) '**************** Start writing seismograms! *****************'
  write(*,*)

   do j=1,rec_num

   !creating the analytical solution

  if (receiver_id == 0) then
    dngix = srd(j)/(srd_diff/(1200/rec_num))
  elseif (receiver_id == 1) then
    dngix = receiver_ix(j) - source_ix
  endif

   !writing seismograms and receiver info to output files

  if (stfpar /= 'de') then
  i=size(greens(:,plot_rec_nr))+size(stf)-1
  conv_ana = 0.
  call conv_fft(analytical(j,:),stf,conv_ana)
  analytical(j,:) = conv_ana(1:nt)
  endif

  call write_seismograms(fname1,greens(:,j),analytical(j,:),srd(j),j,3,dngix,&
                               diff_ana=diff_ana(j))
  enddo

!calculating the corresponding mean dx for every array

if (receiver_id == 0) then
  dx_mean = srd_diff/(srd_diff/(Length/float(ng-1)))
elseif (receiver_id == 1) then
  dx_mean = (x(receiver_ix(1)) - x(source_ix))/float(receiver_ix(1) - source_ix)
endif

  !write additional data to setup file

  setupfile='DATA_TMP/'//trim('usedsetup_'//trim(fname1)//'.m')
  v_min = min_matrix2d(ve)
  if (reg_grid_flag) then
    call write_setup(setupfile,N,ne,mod_id,receiver_id,dt,nt,Length,v_min,&
       tmax,dx_mean,ng,rec_num,source_ix,de(1))
  else
    call write_setup(setupfile,N,ne,mod_id,receiver_id,dt,nt,Length,v_min,&
       tmax,dx_mean,ng,rec_num,source_ix)
  endif

  write(*,*)
  write(*,*) '*************** Finished writing seismograms! ***************'
  write(*,*)

endif

write(*,*)
write(*,*) '*************** Program finished successfully! ***************'
write(*,*)

END PROGRAM elastic_1D

