
MODULE signal_processing

INTERFACE
  SUBROUTINE fft(data,nn,isign)

  integer nn,isign

  double precision data(2*nn)

  END SUBROUTINE
END INTERFACE

INTERFACE
  SUBROUTINE conv(invector1,invector2,outvector)
   implicit none

   double precision, dimension(:) :: invector1,invector2,outvector
   integer n1, n2
  END SUBROUTINE conv
END INTERFACE

INTERFACE
  SUBROUTINE conv_fft(invector1,invector2,outvector)
   implicit none

  double precision, dimension(:), intent(in) :: invector1,invector2
  double precision, dimension(:), intent(out) :: outvector

  END SUBROUTINE conv_fft
END INTERFACE

END MODULE signal_processing


!****************************************************************************
!****************************************************************************

SUBROUTINE fft(data,nn,isign)

!Replaces data(1:2*nn) by its discrete Fourier transform,
!if isign is input as 1; or replaces data(1:2*nn) by nn times
!its inverse discrete Fourier transform, if isign is input as
!1. data is a complex array of length nn or, equivalently,
!a real array of length 2*nn. nn MUST be an integer power of 2
!(this is not checked for!).

INTEGER isign,nn

double precision data(2*nn)
double precision tempi,tempr
DOUBLE PRECISION theta,wi,wpi,wpr,wr,wtemp
!Double precision for the trigonometric recurrences.

INTEGER i,istep,j,m,mmax,n

n=2*nn
j=1

do i=1,n,2 !This is the bit-reversal section of the routine.

  if(j.gt.i)then
  tempr=data(j) !Exchange the two complex numbers.
  tempi=data(j+1)
  data(j)=data(i)
  data(j+1)=data(i+1)
  data(i)=tempr
  data(i+1)=tempi
  endif
  m=nn
1 if ((m.ge.2).and.(j.gt.m)) then
  j=j-m
  m=m/2
  goto 1
  endif
  j=j+m
enddo

mmax=2 !Here begins the Danielson-Lanczos section of the routine.

2 if (n.gt.mmax) then !Outer loop executed log 2 nn times.

  istep=2*mmax
  theta=6.28318530717959d0/(isign*mmax) !Initialize for the trigonometric recurrence.
  wpr=-2.d0*sin(0.5d0*theta)**2
  wpi=sin(theta)
  wr=1.d0
  wi=0.d0
  do m=1,mmax,2 !Here are the two nested inner loops.
    do i=m,n,istep
  j=i+mmax !This is the Danielson-Lanczos formula:
  tempr=(wr)*data(j)-(wi)*data(j+1)
  tempi=(wr)*data(j+1)+(wi)*data(j)
  data(j)=data(i)-tempr
  data(j+1)=data(i+1)-tempi
  data(i)=data(i)+tempr
  data(i+1)=data(i+1)+tempi
    enddo
    wtemp=wr !Trigonometric recurrence.
    wr=wr*wpr-wi*wpi+wr
    wi=wi*wpr+wtemp*wpi+wi
  enddo
  mmax=istep
  goto 2 !Not yet done.
  endif !All done.
END

!******************************************************************************
!******************************************************************************

SUBROUTINE conv(invector1,invector2,outvector)

!performs the convolution of two vectors using the discrete convolution formula
!h(t)= SUMoverTAU f(t-TAU) g(TAU)
!invector1 is the Greens function of the model space
!invector2 is the source-time function

implicit none

   double precision, dimension(:) :: invector1,invector2,outvector
   integer n1, n2, n, i, j

n1 = size(invector1)
n2 = size(invector2)
n = n1 + n2 -1

outvector=0

do i=1,n1
   do j=1,n2-1
      if (i-j >= 0) outvector(i) = outvector(i) + invector1(i-j)*invector2(j)
   enddo
enddo

END SUBROUTINE conv

!******************************************************************************


SUBROUTINE conv_fft(invector1,invector2,outvector)

use signal_processing

implicit none

double precision, dimension(:), intent(in) :: invector1,invector2
double precision, dimension(:), intent(out) :: outvector
real, dimension(:), allocatable :: dummy

double precision, dimension(:), allocatable :: fft_1, fft_2, fft_out

integer i,n,n1,n2,m,nn,fehler

if (size(invector1) >= size(invector2)) then
  n=size(invector1)
else
  n=size(invector2)
endif

n1=size(invector1)
n2=size(invector2)
nn=1
do while (2**nn < n)
nn=nn+1
enddo

m=2**nn

allocate(fft_1(1:2*m),fft_2(1:2*m),fft_out(1:2*m),dummy(1:m),stat=fehler)

fft_1 = 0.
fft_2 = 0.
fft_out = 0.

do i = 1,size(invector1)
fft_1(2*i-1) = (invector1(i))
enddo

do i = 1,size(invector2)
fft_2(2*i-1) = (invector2(i))
enddo

call fft(fft_1,m,1)
call fft(fft_2,m,1)
do i=1,m
fft_out(2*i-1) = fft_1(2*i-1) * fft_2(2*i-1) -  fft_1(2*i) * fft_2(2*i) !real part
fft_out(2*i-1) = fft_out(2*i-1)/dble(m)
fft_out(2*i) = fft_1(2*i-1) * fft_2(2*i) +  fft_1(2*i) * fft_2(2*i-1) !imaginary part
fft_out(2*i) = fft_out(2*i)/dble(m)
enddo

call fft(fft_out,m,-1)

do i=1,m
dummy(i) = dble(fft_out(2*i-1))
enddo

outvector = dummy(1:size(outvector))

END SUBROUTINE conv_fft

